#!/bin/sh

# This script is used to scan for Wi-Fi networks and send them up to the server
# and also to allow for wifi network setup for the gateway

# Source the jshn shell library
. /usr/share/libubox/jshn.sh

# Get the GWID
GWID=$(uci get gateway.general.GWID)

# Create a temporary file for the JSON payload
TEMP_FILE=$(mktemp)

# Function to scan Wi-Fi networks and parse the output
scan_wifi() {
  iwinfo wlan0 scan
}

# Function to translate encryption values
translate_encryption() {
  local enc="$1"
  case "$enc" in
    "WPA2 PSK (CCMP)"|"WPA2 PSK (AES)") echo "psk2";;
    "WPA PSK (TKIP)"|"WPA PSK (CCMP)"|"WPA PSK (AES)") echo "psk";;
    "mixed WPA/WPA2 PSK (TKIP, CCMP)"|"mixed WPA/WPA2 PSK (CCMP)"|"mixed WPA/WPA2 PSK (AES)") echo "psk-mixed+aes";;
    "WPA2 PSK (AES)"|"WPA2 PSK (CCMP)") echo "psk2+aes";;
    "WPA PSK (AES)") echo "psk+aes";;
    "mixed WPA/WPA2 PSK") echo "psk-mixed";;
    "WEP") echo "wep";;
    "none") echo "none";;
    *) echo "$enc";;
  esac
}

# Function to parse the scan result and write to temporary file
parse_wifi_scan() {
  local ssid=""
  local encryption=""

  scan_wifi | while IFS= read -r line; do
    case "$line" in
      *"Cell "*)
        if [ -n "$ssid" ] && [ "$ssid" != "unknown" ] && [ "$ssid" != "ESSID: unknown" ]; then
          translated_enc=$(translate_encryption "$encryption")
          echo "$ssid|$translated_enc" >> "$TEMP_FILE"
        fi
        ssid=""
        encryption=""
        ;;
      *"ESSID: "*) ssid=$(echo "$line" | cut -d '"' -f 2 | xargs) ;;
      *"Encryption: "*) encryption=$(echo "$line" | cut -d ':' -f 2 | xargs) ;;
    esac
  done

  # Add the last network if it exists and is not "unknown"
  if [ -n "$ssid" ] && [ "$ssid" != "unknown" ] && [ "$ssid" != "ESSID: unknown" ]; then
    translated_enc=$(translate_encryption "$encryption")
    echo "$ssid|$translated_enc" >> "$TEMP_FILE"
  fi
}

# Function to create JSON from the temporary file
create_json() {
  json_init
  json_add_array "networks"

  while IFS='|' read -r ssid encryption; do
    json_add_object
    json_add_string "ESSID" "$ssid"
    json_add_string "Encryption" "$encryption"
    json_close_object
  done < "$TEMP_FILE"

  json_close_array
  json_dump > "$TEMP_FILE"
}

# Execute the functions
parse_wifi_scan
create_json

# Post the JSON payload to the REST endpoint
IP=$(curl -k -X POST -H "Content-Type: application/json" -d @"$TEMP_FILE" "https://portal.securetemps.com:8443/wifi/$GWID")

# Check if the response is empty
if [ -z "$IP" ]; then
  echo "Error: No response from the server"
  rm "$TEMP_FILE"
  exit 1
fi

# Load the JSON response
json_load "$IP"
if [ $? -ne 0 ]; then
  echo "Error: Failed to parse JSON response"
  rm "$TEMP_FILE"
  exit 1
fi

# Extract variables from the JSON response
json_get_var ssid ssid
json_get_var host_key host_key
json_get_var host_enc host_enc

# Validate the extracted values
if [ -n "$ssid" ] && [ -n "$host_key" ] && [ -n "$host_enc" ]; then
  PASS=$(echo $host_key | tr '[N-ZA-Mn-za-m]' '[A-Za-z]')
  uci set wireless.sta_0.ssid=$ssid
  uci set wireless.sta_0.disabled=0
  uci set wireless.sta_0.key=$PASS
  uci set wireless.sta_0.encryption=$host_enc
  # uci show wireless.sta_0
  uci commit wireless
  wifi reload
else
  echo "Error: Missing or invalid values for ssid, host_key, or host_enc"
  uci set wireless.sta_0.ssid=
  uci set wireless.sta_0.disabled=1
  uci set wireless.sta_0.key=
  uci set wireless.sta_0.encryption=
  # uci show wireless.sta_0
  uci commit wireless
  wifi reload
fi

# Clean up the temporary file
rm "$TEMP_FILE"

